(* ICFP Programming Contest 2001
 * Copyright (C) 2001 Jean-Christophe Filliatre *)

(*s SML/NG equivalence validator. *)

open Printf
open Interp

let _ = 
  if Array.length Sys.argv == 1 then begin
    prerr_endline "usage: valid file1 [file2]";
    exit 1
  end

let d1 = 
  let c = open_in Sys.argv.(1) in
  let d = Parse.from_channel c in
  close_in c;
  d

let d2 =
  if Array.length Sys.argv > 2 then begin
    let c = open_in Sys.argv.(2) in
    let d = Parse.from_channel c in
    close_in c;
    d
  end else
    Parse.from_stdin ()

let msg s = 
  eprintf "%s" s; flush stderr

let explain_diff dc1 dc2 = 
  eprintf "1:"; Output.dump_char stderr dc1;
  eprintf "2:"; Output.dump_char stderr dc2

let rec compare s1 = parser
  | [< 'c2; s2 >] -> (match s1 with parser
			| [< 'c1; st1 >] -> c1 = c2 && compare st1 s2
			| [< >] -> false)
  | [< >] -> (match s1 with parser
		| [< 'c1; _ >] -> false
		| [< >] -> true)

let _ =
  let i1 = interp_document d1 in
  let i2 = interp_document d2 in
  if compare i1 i2 then begin
    prerr_string "same"; exit 0
  end else begin
    prerr_string "different"; exit 1
  end



